@@ -324,6 +324,7 @@ class BaseSim(ParsObj):
             t (int or str): the time point in the simulation cloesst to the requested date
 
         **Examples**::
+        
             sim.get_t('2015-03-01') # Get the closest timepoint to the specified date
             sim.get_t(3) # Will return 3
             sim.get_t('2015') # Can use strings
@@ -1271,6 +1272,11 @@ class BasePeople(FlexPretty):
         ''' Boolean array of everyone sexually active i.e. past debut '''
         return ((self.age>self.debut) * (self.alive) * (self.level0)).astype(bool)
 
+    @property
+    def is_female_adult(self):
+        ''' Boolean array of everyone eligible for screening '''
+        return ((self.age>self.debut) * (self.is_female) * (self.alive)).astype(bool)
+
     @property
     def is_virgin(self):
         ''' Boolean array of everyone not yet sexually active i.e. pre debut '''
