@@ -337,12 +337,15 @@ class RoutineDelivery(Intervention):
             errormsg = 'Years must be within simulation start and end dates.'
             raise ValueError(errormsg)
 
+        # Adjustment to get the right end point
+        adj_factor = int(1/sim['dt'])-1 if sim['dt']<1 else 1
+
         # Determine the timepoints at which the intervention will be applied
         self.start_point    = sc.findinds(sim.yearvec, self.start_year)[0]
-        self.end_point      = sc.findinds(sim.yearvec, self.end_year)[0] + int(1/sim['dt']) - 1
+        self.end_point      = sc.findinds(sim.yearvec, self.end_year)[0] + adj_factor
         self.years          = sc.inclusiverange(self.start_year, self.end_year)
         self.timepoints     = sc.inclusiverange(self.start_point, self.end_point)
-        self.yearvec        = np.arange(self.start_year, self.end_year + int(1/sim['dt']) - 1, sim['dt'])
+        self.yearvec        = np.arange(self.start_year, self.end_year+adj_factor, sim['dt'])
 
         # Get the probability input into a format compatible with timepoints
         if len(self.years) != len(self.prob):
@@ -354,8 +357,8 @@ class RoutineDelivery(Intervention):
         else:
             self.prob = sc.smoothinterp(self.yearvec, self.years, self.prob, smoothness=0)
 
-        # Lastly, adjust the annual probability by the sim's timestep, if it's an annual probability
-        if self.annual_prob: self.prob = self.prob*sim['dt']
+        # Lastly, adjust the probability by the sim's timestep, if it's an annual probability
+        if self.annual_prob: self.prob = 1-(1-self.prob)**sim['dt']
 
         return
 
@@ -364,10 +367,10 @@ class CampaignDelivery(Intervention):
     '''
     Base class for any intervention that uses campaign delivery; handles interpolation of input years.
     '''
-    def __init__(self, years, interpolate=True, prob=None, annual_prob=True):
+    def __init__(self, years, interpolate=None, prob=None, annual_prob=True):
         self.years = sc.promotetoarray(years)
-        self.interpolate = interpolate # Whether to space the intervention over the year (if true) or do them all at once (if false)
-        self.prob       = sc.promotetoarray(prob)
+        self.interpolate = True if interpolate is None else interpolate
+        self.prob = sc.promotetoarray(prob)
         self.annual_prob = annual_prob
         return
 
@@ -391,7 +394,7 @@ class CampaignDelivery(Intervention):
             raise ValueError(errormsg)
 
         # Lastly, adjust the annual probability by the sim's timestep, if it's an annual probability
-        if self.annual_prob: self.prob = self.prob*sim['dt']
+        if self.annual_prob: self.prob = 1-(1-self.prob)**sim['dt']
 
         return
 
@@ -554,6 +557,7 @@ __all__ += ['BaseVaccination', 'routine_vx', 'campaign_vx']
 class BaseVaccination(Intervention):
     '''
     Base vaccination class for determining who will receive a vaccine.
+
     Args:
          product        (str/Product)   : the vaccine to use
          prob           (float/arr)     : annual probability of eligible population getting vaccinated
@@ -614,7 +618,7 @@ class BaseVaccination(Intervention):
         '''
         Determine who is eligible for vaccination
         '''
-        conditions = np.full(len(sim.people), True, dtype=bool) # Start by assuming everyone is eligible
+        conditions = sim.people.alive # Start by assuming everyone alive is eligible
         if len(self.sex)==1:
             conditions = conditions & (sim.people.sex == self.sex[0]) # Filter by sex
         if self.age_range is not None:
@@ -633,6 +637,7 @@ class BaseVaccination(Intervention):
         # if the timepoint matches one of the requested timepoints.
         if len(self.timepoints)>0 and (sim.t not in self.timepoints): do_apply = False
         else: do_apply = True
+        accept_inds = np.array([])
 
         if do_apply:
 
@@ -642,21 +647,26 @@ class BaseVaccination(Intervention):
                 prob = self.prob
             else: # Get the proportion of people who screen on this timestep
                 prob = self.prob[sc.findinds(self.timepoints, sim.t)[0]]
-            accept_inds     = select_people(eligible_inds, prob=prob)
+            accept_inds = select_people(eligible_inds, prob=prob)
 
             if len(accept_inds):
-                self.product.administer(sim.people, accept_inds) # Actually change people's immunity
-                # Update people's state and dates, as well as results and doses
+                self.product.administer(sim.people, accept_inds) # Administer the product
+
+                # Update people's state and dates
                 sim.people.vaccinated[accept_inds] = True
                 sim.people.date_vaccinated[accept_inds] = sim.t
                 sim.people.doses[accept_inds] += 1
-                idx = int(sim.t / sim.resfreq)
-                new = sim.people.scale_flows(accept_inds)
-                sim.results['new_vaccinated'][:,idx] += new
-                sim.results['new_doses'][idx] += new
-                self.n_products_used[idx] += new
 
-        return
+                # Update results
+                idx = int(sim.t / sim.resfreq)
+                new_vx_inds = hpu.ifalsei(sim.people.vaccinated, accept_inds)  # Figure out people who are getting vaccinated for the first time
+                n_new_doses = sim.people.scale_flows(accept_inds)  # Scale
+                n_new_people = sim.people.scale_flows(new_vx_inds)  # Scale
+                sim.results['new_vaccinated'][:,idx] += n_new_people
+                sim.results['new_doses'][idx] += n_new_doses
+                self.n_products_used[idx] += n_new_doses
+
+        return accept_inds
 
 
     def shrink(self, in_place=True):
@@ -756,11 +766,10 @@ class BaseTest(Intervention):
         '''
         Deliver the diagnostics by finding who's eligible, finding who accepts, and applying the product.
         '''
-        self.outcomes = {k:np.array([], dtype=hpd.default_int) for k in self.product.hierarchy}
         ti = sc.findinds(self.timepoints, sim.t)[0]
-        prob            = self.prob[ti] # Get the proportion of people who will be tested this timestep
-        eligible_inds   = self.check_eligibility(sim) # Check eligibility
-        accept_inds     = select_people(eligible_inds, prob=prob) # Find people who accept
+        prob = self.prob[ti] # Get the proportion of people who will be tested this timestep
+        eligible_inds = self.check_eligibility(sim) # Check eligibility
+        accept_inds = select_people(eligible_inds, prob=prob) # Find people who accept
         if len(accept_inds):
             idx = int(sim.t / sim.resfreq)
             self.n_products_used[idx] += sim.people.scale_flows(accept_inds)
@@ -789,24 +798,36 @@ class BaseScreening(BaseTest):
         females in age range, plus any additional user-defined eligibility, which often includes
         the screening interval.
         '''
-        active_females  = sim.people.is_female & sim.people.is_active
-        in_age_range    = (sim.people.age >= self.age_range[0]) & (sim.people.age <= self.age_range[1])
-        conditions      = (active_females & in_age_range)
+        adult_females   = sim.people.is_female_adult
+        in_age_range    = (sim.people.age >= self.age_range[0]) * (sim.people.age <= self.age_range[1])
+        conditions      = (adult_females * in_age_range).astype(bool)
         if self.eligibility is not None:
             other_eligible  = sc.promotetoarray(self.eligibility(sim))
-            conditions      = conditions & other_eligible
+            conditions      = conditions * other_eligible
         return hpu.true(conditions)
 
     def apply(self, sim):
         '''
         Perform screening by finding who's eligible, finding who accepts, and applying the product.
         '''
+        self.outcomes = {k:np.array([], dtype=hpd.default_int) for k in self.product.hierarchy}
+        accept_inds = np.array([])
         if sim.t in self.timepoints:
             accept_inds = self.deliver(sim)
             sim.people.screened[accept_inds] = True
             sim.people.screens[accept_inds] += 1
             sim.people.date_screened[accept_inds] = sim.t
-        return
+
+            # Store results
+            idx = int(sim.t / sim.resfreq)
+            new_screen_inds = hpu.ifalsei(sim.people.screened, accept_inds)  # Figure out people who are getting screened for the first time
+            n_new_people = sim.people.scale_flows(new_screen_inds)  # Scale
+            n_new_screens = sim.people.scale_flows(accept_inds)  # Scale
+            n_new_screens = sim.people.scale_flows(accept_inds)  # Scale
+            sim.results['new_screened'][idx] += n_new_people
+            sim.results['new_screens'][idx] += n_new_screens
+
+        return accept_inds
 
 
 class BaseTriage(BaseTest):
@@ -823,8 +844,10 @@ class BaseTriage(BaseTest):
         return sc.promotetoarray(self.eligibility(sim))
 
     def apply(self, sim):
-        if sim.t in self.timepoints: _ = self.deliver(sim)
-        return
+        self.outcomes = {k:np.array([], dtype=hpd.default_int) for k in self.product.hierarchy}
+        accept_inds = np.array([])
+        if sim.t in self.timepoints: accept_inds = self.deliver(sim)
+        return accept_inds
 
 
 class routine_screening(BaseScreening, RoutineDelivery):
@@ -963,7 +986,7 @@ class BaseTreatment(Intervention):
         Check people's eligibility for treatment
         '''
         females         = sim.people.is_female
-        in_age_range    = (sim.people.age >= self.age_range[0]) & (sim.people.age <= self.age_range[1])
+        in_age_range    = (sim.people.age >= self.age_range[0]) * (sim.people.age <= self.age_range[1])
         alive           = sim.people.alive
         nocancer        = ~sim.people.cancerous.any(axis=0)
         conditions      = (females * in_age_range * alive * nocancer)
@@ -991,12 +1014,28 @@ class BaseTreatment(Intervention):
         '''
         Perform treatment by getting candidates, checking their eligibility, and then treating them.
         '''
+        # Get indices of who will get treated
         treat_candidates = self.get_candidates(sim) # NB, this needs to be implemented by derived classes
         still_eligible = self.check_eligibility(sim)
         treat_inds = hpu.itruei(still_eligible, treat_candidates)
-        self.outcomes = self.product.administer(sim, treat_inds)
+
+        # Store treatment and dates
+        sim.people.cin_treated[treat_inds] = True
+        sim.people.cin_treatments[treat_inds] += 1
+        sim.people.date_cin_treated[treat_inds] = sim.t
+
+        # Store results
         idx = int(sim.t / sim.resfreq)
+        new_treat_inds = hpu.ifalsei(sim.people.cin_treated, treat_inds)  # Figure out people who are getting radiation for the first time
+        n_new_cin_treatments = sim.people.scale_flows(treat_inds)  # Scale
+        n_new_people = sim.people.scale_flows(new_treat_inds)  # Scale
+        sim.results['new_cin_treated'][idx] += n_new_people
+        sim.results['new_cin_treatments'][idx] += n_new_cin_treatments
+
+        # Administer treatment and store products used
+        self.outcomes = self.product.administer(sim, treat_inds)
         self.n_products_used[idx] += sim.people.scale_flows(treat_inds)
+
         return treat_inds
 
 
@@ -1078,7 +1117,8 @@ class treat_delay(BaseTreatment):
         scheduler, and then will treat anyone scheduled for treatment on this timestep.
         '''
         self.add_to_schedule(sim)
-        BaseTreatment.apply(self, sim)
+        treat_inds = BaseTreatment.apply(self, sim)
+        return treat_inds
 
 
 class BaseTxVx(BaseTreatment):
@@ -1088,27 +1128,52 @@ class BaseTxVx(BaseTreatment):
     def __init__(self, **kwargs):
         BaseTreatment.__init__(self, **kwargs)
 
+
     def deliver(self, sim):
         '''
         Deliver the intervention. This applies on a single timestep, whereas apply() methods
         apply on every timestep and can selectively call this method.
         '''
-        eligible_inds = hpu.true(self.check_eligibility(sim))
-        if len(eligible_inds):
-            accept_inds = select_people(eligible_inds, prob=self.prob[0])  # Select people who accept
-            new = sim.people.scale_flows(accept_inds)
-            if new:
-                self.outcomes = self.product.administer(sim, accept_inds)
-                idx = int(sim.t / sim.resfreq)
+        is_eligible = self.check_eligibility(sim) # Apply general eligiblity
+
+        # Apply extra user-defined eligibility conditions, if given
+        if self.eligibility is not None:
+            extra_conditions = sc.promotetoarray(self.eligibility(sim))
+
+            # Checking self.eligibility() can return either a boolean array of indices. Convert to indices.
+            if (len(extra_conditions) == len(is_eligible)) & (len(extra_conditions) > 0):
+                if sc.checktype(extra_conditions[0], 'bool'):
+                    extra_conditions = hpu.true(extra_conditions)
+
+            # Combine the extra conditions with general eligibility
+            if len(extra_conditions)>0:
+                eligible_inds = hpu.itruei(is_eligible, extra_conditions)  # First make sure they're generally eligible
+            else:
+                eligible_inds = np.array([])
+
+        else:
+            eligible_inds = hpu.true(is_eligible)
+
+        # Get anyone eligible and apply acceptance rates
+        accept_inds = np.array([])
+        if len(eligible_inds): # If so, proceed
+            accept_inds = select_people(eligible_inds, prob=self.prob[0]) # Select people who accept
+            new_vx_inds = hpu.ifalsei(sim.people.tx_vaccinated, accept_inds) # Figure out people who are getting vaccinated for the first time
+            n_new_doses  = sim.people.scale_flows(accept_inds) # Scale
+            n_new_people = sim.people.scale_flows(new_vx_inds) # Scale
+
+            if n_new_doses:
+                self.outcomes = self.product.administer(sim, accept_inds) # Administer
                 sim.people.tx_vaccinated[accept_inds] = True
                 sim.people.date_tx_vaccinated[accept_inds] = sim.t
                 sim.people.txvx_doses[accept_inds] += 1
-                idx = int(sim.t / sim.resfreq)
-                sim.results['new_tx_vaccinated'][idx] += new
-                sim.results['new_txvx_doses'][idx] += new
-                self.n_products_used[idx] += new
 
-        return
+                idx = int(sim.t / sim.resfreq)
+                sim.results['new_tx_vaccinated'][idx] += n_new_people
+                sim.results['new_txvx_doses'][idx] += n_new_doses
+                self.n_products_used[idx] += n_new_doses
+
+        return accept_inds
 
 
 class routine_txvx(BaseTxVx, RoutineDelivery):
@@ -1132,13 +1197,16 @@ class routine_txvx(BaseTxVx, RoutineDelivery):
         BaseTxVx.initialize(self, sim) # Initialize this next - this actually calls BaseTreatment.initialize() and ensures products will be counted
 
     def apply(self, sim):
-        if sim.t in self.timepoints: self.deliver(sim)
+        accept_inds = np.array([])
+        if sim.t in self.timepoints:
+            accept_inds = self.deliver(sim)
+        return accept_inds
 
 
 class campaign_txvx(BaseTxVx, CampaignDelivery):
     '''
     Campaign delivery of therapeutic vaccine - an instance of treat_num combined
-     with campaign delivery. See base classes for a description of input arguments.
+    with campaign delivery. See base classes for a description of input arguments.
     '''
 
     def __init__(self, product=None, prob=None, age_range=None, eligibility=None,
@@ -1151,7 +1219,10 @@ class campaign_txvx(BaseTxVx, CampaignDelivery):
         BaseTxVx.initialize(self, sim) # Initialize this next - this actually calls BaseTreatment.initialize() and ensures products will be counted
 
     def apply(self, sim):
-        if sim.t in self.timepoints: self.deliver(sim)
+        accept_inds = np.array([])
+        if sim.t in self.timepoints:
+            accept_inds = self.deliver(sim)
+        return accept_inds
 
 
 class linked_txvx(BaseTxVx):
@@ -1164,7 +1235,8 @@ class linked_txvx(BaseTxVx):
         BaseTxVx.__init__(self, **kwargs)
 
     def apply(self, sim):
-        BaseTxVx.deliver(self, sim)
+        accept_inds = BaseTxVx.deliver(self, sim)
+        return accept_inds
 
 
 #%% Products
@@ -1216,7 +1288,6 @@ class dx(Product):
                 if self.ng>1: df_filter = df_filter & (self.df.genotype == genotype) # also filter by genotype, if this test is by genotype
                 thisdf = self.df[df_filter] # apply filter to get the results for this state & genotype
                 probs = [thisdf[thisdf.result==result].probability.values[0] for result in self.hierarchy] # Pull out the result probabilities in the order specified by the result hierarchy
-
                 # Sort people into one of the possible result states and then update their overall results (aggregating over genotypes)
                 this_gtype_results = hpu.n_multinomial(probs, len(theseinds))
                 results[theseinds] = np.minimum(this_gtype_results, results[theseinds])
@@ -1278,12 +1349,9 @@ class tx(Product):
                         people[state][g, eff_treat_inds] = False  # People who get treated have their CINs removed
                         people[f'date_{state}'][g, eff_treat_inds] = np.nan
 
-                        # Clear infection for women who clear
-                        people.infectious[g, eff_treat_inds] = False  # People whose HPV clears
-                        people.susceptible[g, eff_treat_inds] = True
-                        people.inactive[g, eff_treat_inds] = False
+                        # Set date of clearance of infection on next timestep
+                        people['date_clearance'][g, eff_treat_inds] = people.t + 1
                         people.dur_infection[g, eff_treat_inds] = (people.t - people.date_infectious[g, eff_treat_inds]) * people.pars['dt']
-                        hpi.update_peak_immunity(people, eff_treat_inds, imm_pars=people.pars, imm_source=g)
 
         tx_successful = np.array(list(set(tx_successful)))
         tx_unsuccessful = np.setdiff1d(inds, tx_successful)
@@ -1327,6 +1395,20 @@ class radiation(Product):
         people = sim.people
         new_dur_cancer = hpu.sample(**self.dur, size=len(inds))
         people.date_dead_cancer[inds] += np.ceil(new_dur_cancer / people.pars['dt'])
+
+        # Store treatment and dates
+        sim.people.cancer_treated[inds] = True
+        sim.people.cancer_treatments[inds] += 1
+        sim.people.date_cancer_treated[inds] = sim.t
+
+        # Store results
+        idx = int(sim.t / sim.resfreq)
+        new_cctreat_inds = hpu.ifalsei(sim.people.cancer_treated, inds)  # Figure out people who are getting radiation for the first time
+        n_new_radiaitons = sim.people.scale_flows(inds)  # Scale
+        n_new_people = sim.people.scale_flows(new_cctreat_inds)  # Scale
+        sim.results['new_cancer_treated'][idx] += n_new_people
+        sim.results['new_cancer_treatments'][idx] += n_new_radiaitons
+
         return inds
 
 
