@@ -68,8 +68,8 @@ class Sim(hpb.BaseSim):
 
     def load_hiv_data(self, location=None, hiv_datafile=None, art_datafile=None, **kwargs):
         ''' Load any data files that are used to create additional parameters, if provided '''
-        self.hiv_pars = sc.objdict()
-        self.hiv_pars.infection_rates, self.hiv_pars.art_adherence = hppar.get_hiv_pars(location=location, hiv_datafile=hiv_datafile, art_datafile=art_datafile)
+        self.hiv_data = sc.objdict()
+        self.hiv_data.infection_rates, self.hiv_data.art_adherence = hppar.get_hiv_data(location=location, hiv_datafile=hiv_datafile, art_datafile=art_datafile)
         return
 
 
@@ -229,6 +229,12 @@ class Sim(hpb.BaseSim):
             errormsg = f'Verbose argument should be either "brief", -1, or a float, not {type(self["verbose"])} "{self["verbose"]}"'
             raise ValueError(errormsg)
 
+        # Handle HIV
+        if self['model_hiv']:
+            if self.hiv_data['infection_rates'] is None or self.hiv_data['art_adherence'] is None:
+                errormsg = 'Data on HIV infection rates and ART adherence must be provided if model_hiv is True.'
+                raise ValueError(errormsg)
+
         return
 
 
@@ -304,42 +310,59 @@ class Sim(hpb.BaseSim):
 
 
     def init_genotypes(self):
-        ''' Initialize the genotypes '''
+        ''' Initialize the genotype parameters '''
         if self._orig_pars and 'genotypes' in self._orig_pars:
             self['genotypes'] = self._orig_pars.pop('genotypes')  # Restore
 
-        genotype_options = hppar.get_genotype_pars().keys()
+        default_gpars   = hppar.get_genotype_pars()
+        user_gpars      = sc.dcp(self['genotype_pars'])
+        self['genotype_pars'] = sc.objdict()
+
+        # Handle special input cases
         if self['genotypes'] == 'all':
-            self['genotypes'] = genotype_options
-
-        for i, g in enumerate(self['genotypes']):
-
-            # Genotypes can be provided in different formats. Here we convert them to hpv.genotype objects
-            if sc.isnumber(g): g = f'hpv{g}' # Convert e.g. 16 to hpv16
-            if sc.checktype(g,str):
-                if not g in genotype_options:
-                    errormsg = f'Genotype {i} ({g}) is not one of the inbuilt options.'
-                    raise ValueError(errormsg)
-                else:
-                    g = hpimm.genotype(g)
-
-            # Initialize genotypes
-            if isinstance(g, hpimm.genotype):
-                if not g.initialized:
-                    g.initialize(self)
-            else:  # pragma: no cover
-                errormsg = f'Cannot understand genotype {i} ({g}). Please provide it as an integer, string, or hpv.genotype object.'
-                raise TypeError(errormsg)
-
+            self['genotypes'] = default_gpars.keys()
         if not len(self['genotypes']):
             print('No genotypes provided, will assume only simulating HPV16 by default')
-            hpv16 = hpimm.genotype('hpv16')
-            hpv16.initialize(self)
-            self['genotypes'] = [hpv16]
+            self['genotypes'] = ['hpv16']
+
+        # Loop over genotypes
+        for i, g in enumerate(self['genotypes']):
+
+            # Standardize format of genotype inputs
+            if sc.isnumber(g): g = f'hpv{g}' # Convert e.g. 16 to hpv16
+            if sc.checktype(g,str):
+                if not g in default_gpars.keys():
+                    errormsg = f'Genotype {i} ({g}) is not one of the inbuilt options.'
+                    raise ValueError(errormsg)
+            else:
+                errormsg = f'Format {type(g)} is not understood.'
+                raise ValueError(errormsg)
+
+            # Add to genotype_par dict
+            self['genotype_pars'][g] = default_gpars[g]
+            self['genotype_map'][i] = g
+
+        # Loop over user-supplied genotype parameters that can overwrite values
+        if len(user_gpars):
+            for g,gpars in user_gpars.items():
+
+                # Standardize format of genotype inputs
+                if sc.isnumber(g): g = f'hpv{g}'  # Convert e.g. 16 to hpv16
+                if sc.checktype(g, str):
+                    if not g in self['genotype_pars'].keys():
+                        errormsg = f'Parameters provided for genotype {g}, but it is not in the sim.'
+                        raise ValueError(errormsg)
+                    else:
+                        for gparname,gparval in gpars.items():
+                            if gparname in self['genotype_pars'][g].keys():
+                                printmsg = f"Resetting parameter '{gparname}' from {self['genotype_pars'][g][gparname]} to {gparval} for genotype {g}"
+                                sc.printv(printmsg, 1, self['verbose'])
+                                self['genotype_pars'][g][gparname] = gparval
+                            else:
+                                errormsg = f"Parameter {gparname} does not exist for genotype {g}"
+                                raise ValueError(errormsg)
 
         len_pars = len(self['genotype_pars'])
-        len_map = len(self['genotype_map'])
-        assert len_pars == len_map, f"genotype_pars and genotype_map must be the same length, but they're not: {len_pars} ≠ {len_map}"
         self['n_genotypes'] = len_pars  # Each genotype has an entry in genotype_pars
 
         # Set the number of immunity sources
@@ -447,6 +470,20 @@ class Sim(hpb.BaseSim):
         results['cum_txvx_doses'] = init_res('Cumulative therapeutic vaccine doses')
         results['cum_tx_vaccinated'] = init_res('Total received therapeutic vaccine')
 
+        # Screen & treat results
+        results['new_screens'] = init_res('New screens')
+        results['new_screened'] = init_res('Newly screened')
+        results['new_cin_treatments'] = init_res('New CIN treatments')
+        results['new_cin_treated'] = init_res('Newly treated for CINs')
+        results['new_cancer_treatments'] = init_res('New cancer treatments')
+        results['new_cancer_treated'] = init_res('Newly treated for cancer')
+        results['cum_screens'] = init_res('Cumulative screens')
+        results['cum_screened'] = init_res('Cumulative number screened')
+        results['cum_cin_treatments'] = init_res('Cumulative CIN treatments')
+        results['cum_cin_treated'] = init_res('Cumulative number treated for CINs')
+        results['cum_cancer_treatments'] = init_res('Cumulative cancer treatments')
+        results['cum_cancer_treated'] = init_res('Cumulative number treated for cancer')
+
         # Additional cancer results
         results['detected_cancer_incidence'] = init_res('Detected cancer incidence', color='#fcba03')
         results['cancer_mortality'] = init_res('Cancer mortality')
@@ -518,7 +555,7 @@ class Sim(hpb.BaseSim):
         self['ms_agent_ratio'] = int(self['ms_agent_ratio'])
         
         # Finish initialization
-        self.people.initialize(sim_pars=self.pars, hiv_pars=self.hiv_pars) # Fully initialize the people
+        self.people.initialize(sim_pars=self.pars, hiv_pars=self.hiv_data) # Fully initialize the people
         self.reset_layer_pars(force=False) # Ensure that layer keys match the loaded population
         if init_states:
             init_hpv_prev = sc.dcp(self['init_hpv_prev'])
@@ -539,17 +576,13 @@ class Sim(hpb.BaseSim):
             if isinstance(intervention, hpi.Intervention):
                 intervention.initialize(self)
 
-        # Set the number of immunity sources
-        # self['n_imm_sources'] += len([x for x in self['interventions'] if isinstance(x, hpi.BaseVaccination)])
-        # self['n_imm_sources'] = self['immunity'].shape[0]
         return
 
-
-    def finalize_interventions(self):
-        for intervention in self['interventions']:
-            if isinstance(intervention, hpi.Intervention):
-                if hasattr(intervention,'n_products_used'):
-                    self.results[f'resources_{intervention.label}'] = intervention.n_products_used
+    # def finalize_interventions(self):
+    #     for intervention in self['interventions']:
+    #         if isinstance(intervention, hpi.Intervention):
+    #             if hasattr(intervention,'n_products_used'):
+    #                 self.results[f'resources_{intervention.label}'] = intervention.n_products_used
 
 
     def init_analyzers(self):
@@ -780,6 +813,10 @@ class Sim(hpb.BaseSim):
                     # For n_total_susceptible, we get the total number of infections that could theoretically happen in the population, which can be greater than the population size
                     self.results[f'n_total_{key}'][idx] = people.count(key)
 
+            # Create stocks of interventions
+            for key in [state.name for state in hpd.PeopleMeta.intv_states]:
+                self.results[f'n_{key}'][idx] = people.count(key)
+
             # Count total hiv infections
             self.results['n_hiv'][idx] = people.count('hiv')
 
@@ -885,7 +922,7 @@ class Sim(hpb.BaseSim):
 
         # Finalize analyzers and interventions
         self.finalize_analyzers()
-        self.finalize_interventions()
+        # self.finalize_interventions()
 
         # Final settings
         self.results_ready = True # Set this first so self.summary() knows to print the results
@@ -1034,8 +1071,16 @@ class Sim(hpb.BaseSim):
         string = f'Simulation{labelstr} summary:\n'
         for key in self.result_keys():
             if full or key.startswith('total') and 'by_sex' not in key:
-                val = np.round(summary[key])
-                string += f'   {val:10,.0f} {self.results[key].name.lower()}\n'.replace(',', sep) # Use replace since it's more flexible
+                val = summary[key]
+                printval = f'   {val:10,.0f} '
+                label = self.results[key].name.lower().replace(',', sep)
+                if 'incidence' in key or 'prevalence' in key:
+                    if key in ['total_hpv_prevalence', 'total_hpv_incidence']:
+                        printval = f'   {val*100:10.2f} '
+                        label += ' (/100)'
+                    else:
+                        label += ' (/100,000)'
+                string += printval + label + '\n'
 
         # Print or return string
         if not output:
